"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var promisify = require("util.promisify");
var zlib = require("zlib");
var path = require("path");
var readFileAsync = promisify(fs.readFile);
var inflateAsync = promisify(zlib.inflate);
var Commit = (function () {
    function Commit(gitDir, hash, data) {
        this.gitDir = gitDir;
        this.hash = hash;
        this.data = data;
        var _a = data.split('\u0000'), head = _a[0], rest = _a.slice(1);
        var _b = head.split(/\s/), type = _b[0], size = _b[1];
        this.size = +size;
        this.body = rest.join('\u0000');
        var m = this.body.match(/^parent\s[a-f0-9]{40}/gm);
        this.parentHashes = m ? m.map(function (s) { return s.split(/\s/).pop(); }) : [];
    }
    Object.defineProperty(Commit.prototype, "hasParents", {
        get: function () {
            return !!this.parentHashes.length;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Commit.prototype, "isMergeCommit", {
        get: function () {
            return this.parentHashes.length >= 2;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Commit.prototype, "baseParentHash", {
        get: function () {
            return this.parentHashes[0];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Commit.prototype, "mergedParentHashes", {
        get: function () {
            return this.parentHashes.slice(1);
        },
        enumerable: true,
        configurable: true
    });
    Commit.prototype.walk = function (parentHash) {
        if (parentHash === void 0) { parentHash = this.baseParentHash; }
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, Commit.readCommit(this.gitDir, parentHash)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    Commit.prototype.walkSync = function (parentHash) {
        if (parentHash === void 0) { parentHash = this.baseParentHash; }
        return Commit.readCommitSync(this.gitDir, parentHash);
    };
    Commit.readCommit = function (gitDir, hash) {
        return __awaiter(this, void 0, void 0, function () {
            var deflatedData, data;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, readFileAsync(getObjectPath(gitDir, hash))];
                    case 1:
                        deflatedData = _a.sent();
                        return [4 /*yield*/, inflateAsync(deflatedData)];
                    case 2:
                        data = (_a.sent()).toString('utf8');
                        return [2 /*return*/, new Commit(gitDir, hash, data)];
                }
            });
        });
    };
    Commit.readCommitSync = function (gitDir, hash) {
        var deflatedData = fs.readFileSync(getObjectPath(gitDir, hash));
        var data = zlib.inflateSync(deflatedData).toString('utf8');
        return new Commit(gitDir, hash, data);
    };
    return Commit;
}());
exports.Commit = Commit;
function getObjectPath(gitDir, hash) {
    return path.join(gitDir, 'objects', hash.replace(/^(.{2})(.{38})$/, "$1" + path.sep + "$2"));
}
